import uuid
from typing import Any, Tuple

import click

from gcs_cli.helpers import record_print_result_message, requires_login
from gcs_cli.parsing import globus_command
from gcs_cli.sdk_client.data import BatchDocument
from gcs_cli.services.gcs import get_client


@globus_command(
    "batch-delete",
    short_help="Delete multiple guest collections",
    disable_options=["format"],
)
@click.argument(
    "collection_id",
    metavar="COLLECTION_ID",
    type=click.UUID,
    nargs=-1,
    required=True,
)
@requires_login
def collection_batch_delete(collection_id: Tuple[uuid.UUID], **kwargs: Any) -> None:
    """
    Delete an existing Collection. This requires the administrator role on the
    Endpoint.
    """
    if len(collection_id) > 50:
        raise click.UsageError("Maximum batch size of 50")
    client = get_client()
    data = BatchDocument(ids=collection_id)
    res = client.batch_collection_delete(data=data)
    record_print_result_message(res)
