from __future__ import annotations

from typing import Optional

import click

from globus_sdk.paging import Paginator

from gcs_cli.helpers import default_principal_resolver, requires_login
from gcs_cli.parsing import globus_command, page_size_opt
from gcs_cli.prettyprint import FORMAT_TEXT_TABLE, FieldSpec, formatted_print
from gcs_cli.services.gcs import get_client

FIELDS: FieldSpec = [
    ("Role ID", "id"),
    ("Role", "role"),
    ("Principal", default_principal_resolver.field),
]


@globus_command("list", short_help="List Endpoint level Roles.")
@click.option("--my-roles", is_flag=True, help="Only list roles which apply to you")
@page_size_opt
@requires_login
def role_list(my_roles: bool = False, page_size: int | None = None) -> None:
    """
    List Endpoint level Roles on the logged-in Endpoint.
    """
    include: Optional[str] = None
    if not my_roles:
        include = "all_roles"

    client = get_client()
    params = {}
    if page_size:
        params["page_size"] = page_size
    res: Paginator = client.paginated.get_role_list(include=include, **params)

    formatted_print(
        res,
        text_format=FORMAT_TEXT_TABLE,
        fields=FIELDS,
        page_callback=default_principal_resolver.page_callback,
    )
