from typing import Any, Optional

from gcs_cli.commands.storage_gateway._common import make_create_command
from gcs_cli.parsing import (
    dropbox_options,
    globus_command,
    storage_gateway_create_and_update_params,
)
from gcs_cli.sdk_client.data import DropboxStoragePolicies, StorageGatewayDocument
from gcs_cli.services.gcs import DROPBOX_CONNECTOR_ID


@globus_command("dropbox")
@storage_gateway_create_and_update_params(create=True)
@dropbox_options(create=True)
@make_create_command
def dropbox_create(
    *,
    dropbox_client_id: Optional[str],
    dropbox_client_secret: Optional[str],
    dropbox_user_api_rate_limit: Optional[int],
    dropbox_allow_any_account: Optional[bool],
    **kwargs: Any
) -> StorageGatewayDocument:
    """Create a new Dropbox Storage Gateway"""
    policies = DropboxStoragePolicies(
        client_id=dropbox_client_id,
        secret=dropbox_client_secret,
        user_api_rate_limit=dropbox_user_api_rate_limit,
        allow_any_account=dropbox_allow_any_account,
    )
    return StorageGatewayDocument(
        connector_id=DROPBOX_CONNECTOR_ID, policies=policies, **kwargs
    )
