import uuid
from typing import Any, Iterable, Optional, Tuple

from gcs_cli.commands.storage_gateway._common import make_update_command
from gcs_cli.parsing import (
    ceph_options,
    globus_command,
    storage_gateway_create_and_update_params,
    storage_gateway_id_arg,
)
from gcs_cli.sdk_client.data import CephStoragePolicies, StorageGatewayDocument
from gcs_cli.services.gcs import CEPH_CONNECTOR_ID


@globus_command("ceph", short_help="Update a Ceph Storage Gateway")
@storage_gateway_id_arg
@storage_gateway_create_and_update_params(create=False)
@ceph_options
@make_update_command
def ceph_update(
    storage_gateway_id: uuid.UUID,
    *,
    s3_endpoint: Optional[str],
    bucket: Optional[Iterable[str]],
    ceph_admin_key_id: Optional[str],
    ceph_admin_secret_key: Optional[str],
    **kwargs: Any
) -> Tuple[uuid.UUID, StorageGatewayDocument]:
    """
    Update an existing Ceph Storage Gateway.
    Any options given will replace existing values in the storage gateway's definition.
    Any options not given will retain their existing values.
    """
    policies = None
    if any(
        x is not None
        for x in (s3_endpoint, bucket, ceph_admin_key_id, ceph_admin_secret_key)
    ):
        policies = CephStoragePolicies(
            s3_endpoint=s3_endpoint,
            s3_buckets=bucket,
            ceph_admin_key_id=ceph_admin_key_id,
            ceph_admin_secret_key=ceph_admin_secret_key,
        )

    return (
        storage_gateway_id,
        StorageGatewayDocument(
            connector_id=CEPH_CONNECTOR_ID, policies=policies, **kwargs
        ),
    )
