import click

from globus_sdk import AuthAPIError

from gcs_cli.helpers import print_command_hint, requires_login
from gcs_cli.parsing import globus_command
from gcs_cli.prettyprint import formatted_print
from gcs_cli.services.auth import get_client as get_auth_client


@globus_command("whoami", disable_options=["map_http_status", "explicit_host"])
@requires_login
def whoami_command() -> None:
    """Show the currently logged-in user's identities"""
    client = get_auth_client()

    print_command_hint(
        "For information on which identities are in session see\n"
        "  globus-connect-server session show\n"
    )

    # get userinfo from auth.
    # if we get back an error the user likely needs to log in again
    try:
        res = client.oauth2_userinfo()
    except AuthAPIError:
        click.echo(
            "Unable to get user information. Please try logging in again.", err=True
        )
        click.get_current_context().exit(1)

    formatted_print(
        res["identity_set"],
        fields=[
            ("Username", "username"),
            ("Name", "name"),
            ("ID", "sub"),
            ("Email", "email"),
        ],
        non_gcs_response=True,
        list_field_name="sub",
    )
