import os
import re
from typing import Dict, List, Optional, Union

from typing_extensions import TypedDict

from .gridftp_option_parser import GridFTPOptionParser


class CalloutDict(TypedDict):
    raw: str
    callout_type: str
    mandatory: bool
    library: str
    symbol: str
    environment: Dict[str, str]


class GsiAuthzParser:
    def parse(
        self,
        authz_config: Optional[Union[str, os.PathLike]],
        gridmap: Optional[Union[str, os.PathLike]],
        gridftp_config: Optional[GridFTPOptionParser],
    ) -> Optional[List[Union[CalloutDict, None]]]:
        callouts: List[Optional[CalloutDict]] = []
        if authz_config:
            with open(authz_config, "r") as f:
                for line in f:
                    line = re.sub(r"^#.*", "", line)
                    line = re.sub(r"^\s*", "", line)
                    if not line:
                        continue
                    mandatory = line[0] == "|"
                    if mandatory:
                        line = line[1:]
                    callout_type, library, symbol, *rest = line.split(maxsplit=3)
                    callout_env: Dict[str, str] = {}

                    if gridftp_config is not None:
                        callout_env.update(gridftp_config.get_environment())

                    if rest and "ENV:" in rest[0]:
                        env = rest[0].split("ENV:", 1)[1]
                        while env:
                            match = re.match(
                                r'^\s*(?P<envar>[^=]+)=(?P<enval>[^" \t\n]+|["][^"]*["])',
                                env,
                            )
                            if match:
                                envar = match.group("envar")
                                enval = match.group("enval")
                                if enval.startswith('"'):
                                    enval = enval[1:-1]
                                callout_env[envar] = enval
                                env = env[match.span()[1] :]
                            else:
                                env = ""
                    callouts.append(
                        {
                            "raw": line.strip(),
                            "callout_type": callout_type,
                            "mandatory": mandatory,
                            "library": library,
                            "symbol": symbol,
                            "environment": callout_env,
                        }
                    )
        if gridmap:
            callouts.append(None)
        if callouts:
            return callouts
        return None
