import marshmallow as ma

from ..api.data_type import FormSchema
from ..cleanup_uuid_mixin import CleanupUUIDMixin
from ..patch_schema import create_patch_schema


class OAuthUserCredentialForm(FormSchema, CleanupUUIDMixin):
    """
    Credential post form
    """

    # Required fields
    identity_id = ma.fields.UUID(
        required=True,
        description="Globus Auth identity id that this credential is associated with",
    )
    # Dropbox has a 500 byte state limit, and this is the only state arg
    # with a variable len
    redirect_uri = ma.fields.Url(
        required=True,
        description=(
            "URL to redirect to once the credential registration flow is complete. "
            "This should be a maximum of 220 characters to avoid conflicts with "
            "connector state limits."
        ),
    )
    storage_gateway = ma.fields.UUID(
        required=True, description="Storage gateway to associate the credential with"
    )
    login_hint = ma.fields.Str(
        required=False, description="Mapped account username on the storage gateway"
    )
    # I don't know why collections app sends this
    access_token = ma.fields.String(description="Unused")
    uuid_fields = ["identity_id", "storage_gateway"]


OAuthUserCredentialForm.public_schema = OAuthUserCredentialForm(
    load_only=OAuthUserCredentialForm.private
)
OAuthUserCredentialForm.private_schema = OAuthUserCredentialForm()
OAuthUserCredentialForm.patch_schema = create_patch_schema(OAuthUserCredentialForm)

user_credential_form_post_schema = OAuthUserCredentialForm.public_schema
