import marshmallow as ma

from ..api import (
    Domain,
    data_type,
    data_type_schema,
)
from ..api.endpoint import (
    Endpoint_1_0_0,
)
from .oidc_registration import OIDCRegistration
from .oidc_server import OIDCServer


@data_type("endpoint_config#1.0.0", schema_base_name="EndpointConfig", private=True)
class EndpointConfig_1_0_0_Schema(Endpoint_1_0_0):
    metadata = {"x-private": True}
    # enable sequential random-looking domains
    domain_index = ma.fields.Int()

    oauth_state_key = ma.fields.Str(
        metadata={
            "description": "Secret to encode and decode the OAuth state variable"
        },
    )

    # fingerprint used as gcs_private_version_json
    fingerprint = ma.fields.Raw(required=False)


@data_type("endpoint_config#1.1.0", schema_base_name="EndpointConfig", private=True)
class EndpointConfig_1_1_0_Schema(EndpointConfig_1_0_0_Schema):
    oidc_server = ma.fields.Nested(
        OIDCServer.private_schema, required=False, allow_none=True
    )

    domain = ma.fields.Nested(Domain.private_schema, required=False, allow_none=True)


@data_type("endpoint_config#1.2.0", schema_base_name="EndpointConfig", private=True)
class EndpointConfig_1_2_0_Schema(EndpointConfig_1_1_0_Schema):
    oidc_registration = ma.fields.Nested(
        OIDCRegistration.private_schema, required=False, allow_none=True
    )


@data_type("endpoint_config#1.3.0", schema_base_name="EndpointConfig", private=True)
class EndpointConfig_1_3_0_Schema(EndpointConfig_1_2_0_Schema):
    gridftp_control_channel_port = ma.fields.Integer(
        min_inclusive=1, max_inclusive=65535, required=False, allow_none=True
    )


@data_type("endpoint_config#1.4.0", schema_base_name="EndpointConfig", private=True)
class EndpointConfig_1_4_0_Schema(EndpointConfig_1_3_0_Schema):
    earliest_last_access = ma.fields.Date(
        allow_none=False,
        description="""
        Earliest date when this endpoint began tracking last_access for
        collections
        """,
    )


EndpointConfig = data_type_schema.EndpointConfig
