import uuid
from typing import Union

import marshmallow as ma
from typing_extensions import Literal, TypedDict

from globus.manager.constants import PrincipalType, RoleType

from ..api import Schema, data_type, data_type_schema
from ..enum_field import EnumField


class RoleConfigDict(TypedDict, total=False):
    DATA_TYPE: Literal["role_config#1.0.0"]
    id: uuid.UUID
    principal_type: PrincipalType
    identity_id: uuid.UUID
    role: RoleType
    resource: Union[None, uuid.UUID]
    primary_role: bool


@data_type("role_config#1.0.0", private=True)
class RoleConfig_1_0_0(Schema):
    id = ma.fields.UUID()
    principal_type = EnumField(PrincipalType)
    identity_id = ma.fields.UUID()
    role = EnumField(RoleType)
    resource = ma.fields.UUID(allow_none=True)
    primary_role = ma.fields.Boolean()

    def load(self, *args, **kwargs) -> RoleConfigDict:
        return super().load(*args, **kwargs)


RoleConfig = data_type_schema.RoleConfig
