import enum
import typing as t

import marshmallow as ma


class EnumField(ma.fields.Field):
    def __init__(self, enum_class, *args, **kwargs):
        if not issubclass(enum_class, enum.Enum):
            raise ValueError("{} is not an enum".format(enum_class.__name__))
        self.enum_class = enum_class
        super().__init__(*args, **kwargs)

    def _serialize(self, value, attr, obj, **kwargs):
        if isinstance(value, self.enum_class):
            return value.name
        return value

    def _deserialize(self, value, attr, data, **kwargs):
        try:
            return getattr(self.enum_class, value)
        except Exception as e:
            raise ma.ValidationError(
                "{} not a value for {}".format(value, self.enum_class.__name__), e
            )

    def _jsonschema_type_mapping(self) -> t.Dict[str, t.Any]:
        type_mapping = {"type": "string", "enum": [e.name for e in self.enum_class]}
        metadata = self.metadata.get("metadata", {})
        metadata.update(self.metadata)
        for md_key, md_val in metadata.items():
            if md_key in ("metadata", "name"):
                continue
            type_mapping[md_key] = md_val
        return type_mapping
