import marshmallow as ma


class CommonCollectionSchema(ma.Schema):
    display_name = ma.fields.String(
        required=True, validate=[ma.validate.Length(max=128)]
    )
    default_directory = ma.fields.Str()
    tlsftp_server = ma.fields.Url(schemes=["tlsftp"], attribute="tlsftp_url")
    https_server = ma.fields.Url(schemes=["https"], attribute="https_url")
    organization = ma.fields.String(
        default=None, validate=[ma.validate.Length(max=1024)]
    )
    department = ma.fields.String(default=None, validate=[ma.validate.Length(max=1024)])
    keywords = ma.fields.Method(
        serialize="dump_keywords", allow_none=True, default=None
    )
    description = ma.fields.Str(default=None, validate=[ma.validate.Length(max=4096)])
    contact_email = ma.fields.Email(default=None)
    contact_info = ma.fields.Str(default=None, validate=[ma.validate.Length(max=4096)])
    info_link = ma.fields.Url(default=None)
    force_encryption = ma.fields.Boolean(required=True, allow_none=False)
    disable_verify = ma.fields.Boolean(required=True, allow_none=False, default=False)
    force_verify = ma.fields.Boolean(required=True, allow_none=False, default=False)
    authentication_timeout_mins = ma.fields.Int(required=True, allow_none=False)
    require_mfa = ma.fields.Boolean(
        required=True, allow_none=False, default=False, data_key="mfa_required"
    )
    high_assurance = ma.fields.Boolean(required=True, allow_none=False)
    user_message = ma.fields.Str(
        required=False,
        allow_none=True,
        default=None,
        validate=[ma.validate.Length(max=64)],
    )
    user_message_link = ma.fields.Url(
        required=False, allow_none=True, default=None, schemes=["http", "https"]
    )

    def dump_keywords(self, obj):
        keywords = ma.utils.get_value(obj, "keywords")
        if keywords is None or keywords == [] or keywords is ma.missing:
            return None
        else:
            return ",".join(keywords)

    @ma.post_dump(pass_original=True)
    def redact_default_directory(self, data, original=None, **kwargs):
        data["default_directory"] = "/{server_default}/"
        return data
