import click

from gcs_cli.parsing.command_decorator import globus_command

from .utils.dns_patterns import verify_domain_name_option, verify_hostname_option


@globus_command(
    name="create",
    short_help="Create an OIDC server for this endpoint",
    disable_options=["format", "explicit_host"],
    help="""\
The OIDC create command creates an OIDC server configuration for this endpoint.

Once configured, install the OIDC server on each node by running
`globus-connect-server oidc setup`.

== EXAMPLE

globus-connect-server oidc create

""",
)
@click.option("--display-name", help="Display name for the OIDC server", required=True)
@click.option(
    "--support-contact", help="Support contact name for the OIDC server", required=True
)
@click.option(
    "--support-email",
    help="Support contact email address for the OIDC server",
    required=True,
)
@click.option(
    "--pam-service",
    "-p",
    help="Specify the PAM service module to use for authentication. Default is 'login'",
    default="login",
)
@click.option(
    "--server-name",
    metavar="DOMAIN_NAME",
    help="Fully-qualified domain name for the OIDC service",
    callback=verify_domain_name_option,
)
@click.option(
    "--certificate-path",
    metavar="PATH",
    help="Path to the certificate for the virtual host for the OIDC service",
)
@click.option(
    "--certificate-chain-path",
    metavar="PATH",
    help="Path to the certificate chain for the virtual host for the OIDC service",
)
@click.option(
    "--private-key-path",
    metavar="PATH",
    help="Path to the private key for the virtual host for the OIDC service",
)
@click.option(
    "--manage-certificate-and-key",
    is_flag=True,
    default=False,
    help="""\
Encrypt and synchronize the certificate and key between data transfer nodes; if
not passed, you are responsible for ensuring that the certificate, chain, and
key files are present on each data transfer node
""",
)
@click.option(
    "--quickstart-server-name",
    help="""\
DNS label to use for configuring the OIDC server to use a subdomain of
the endpoint's domain.
""",
    callback=verify_hostname_option,
)
@click.pass_context
def oidc_create(
    ctx,
    display_name,
    support_contact,
    support_email,
    pam_service,
    server_name,
    certificate_path,
    certificate_chain_path,
    private_key_path,
    manage_certificate_and_key,
    quickstart_server_name,
) -> None:
    from .oidc_create import oidc_create as oc

    oc(
        ctx,
        display_name,
        support_contact,
        support_email,
        pam_service,
        server_name,
        certificate_path,
        certificate_chain_path,
        private_key_path,
        manage_certificate_and_key,
        quickstart_server_name,
    )
